//+------------------------------------------------------------------+
//|                                         FX Steady Growth .mq4    |
//+------------------------------------------------------------------+

#define MagicNumber  20050555

extern bool   MoneyManagement    = false;     
extern double LotsPercent        = 2;       
extern double ManualLotSize      = 0.1;      
extern double StopLoss           = 55;
extern double TakeProfit         = 800;
extern double MovingPeriod       = 21;
extern double MovingShift        = 12;
extern int    TrailingStop       = 65;
extern int    Slippage           = 3;

double point;

//+------------------------------------------------------------------+
int init()
  {
//----
   if(BrokerIs5Digit())
   {
      StopLoss*=10;
      TakeProfit*=10;
      Slippage*=10;
      TrailingStop*=10;
   }
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
  double MinLot=NormalizeDouble(MarketInfo(Symbol(),MODE_MINLOT),2);
  double MaxLot=NormalizeDouble(MarketInfo(Symbol(),MODE_MAXLOT),2);
  double LotStep=NormalizeDouble(MarketInfo(Symbol(),MODE_LOTSTEP),2);
  double FullMargin=MarketInfo(Symbol(),MODE_MARGINREQUIRED);
  double LotSize,Lot;
  int LotDigit;
  string MM="";
  
  //////////////////////////////  
  if(MoneyManagement==true)
  {
  LotSize=(AccountBalance()/100000)*LotsPercent;
  }
  ////////////////////////////////////////////////// 
  if(MoneyManagement==false) LotSize=ManualLotSize;
  /////////////////////////////////////////////
  if(LotSize<MinLot) LotSize=MinLot;
  if(LotSize>MaxLot) LotSize=MaxLot;
  /////////////////////////////////////////////
  if(LotStep==1) LotDigit=0;
  if(LotStep==0.1) LotDigit=1;
  if(LotStep==0.01) LotDigit=2;
  
  //////////////////////////////////////////
  Lot= MathFloor(LotSize/LotStep)*LotStep;
   return(Lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
{
   double ma;
   int    res;
   
//---- go trading only for first tiks of new bar
   static datetime Time0; if (Time0 == Time[0]) return(0); Time0 = Time[0];
//---- get Moving Average 
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
//---- sell conditions

   if(Open[1]>ma && Close[1]<ma)  
   {
      OpenSell("");
      return;
   }
//---- buy conditions
   if(Open[1]<ma && Close[1]>ma)  
   {
      OpenBuy("");
      return;
   }
}
//+------------------------------------------------------------------+
//| Trailing Stop function                                           |
//+------------------------------------------------------------------+
void Trailing_Stop()
{
   for(int cnt = OrdersTotal() - 1; cnt >= 0; cnt--)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      //----
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
      {       
	  	   if(OrderType() == OP_BUY)
	  	   {
	  		   if(TrailingStop > 0) 
	  		   {
			      if(Bid - OrderOpenPrice() >= TrailingStop*Point)
			      {
	               if(OrderStopLoss() < (Bid - Point*TrailingStop)) 
                  {					   
				         OrderModify(OrderTicket(), OrderOpenPrice(), Bid - Point*TrailingStop, 0, 0, Yellow);
				         return(0);
	  		         }
	  		      }
	  		   }
	  		}
	  	   if(OrderType() == OP_SELL) 
	  	   {
	  	      if(TrailingStop > 0) 
	  	      {
		         if(OrderOpenPrice() - Ask >= TrailingStop*Point)
		         {
		            if(OrderStopLoss() > (Ask + Point*TrailingStop))
		            {
		               OrderModify(OrderTicket(), OrderOpenPrice(), Ask + Point*TrailingStop, 0, 0, Purple);
		               return(0);
		            }
   	         }
            } 
         }
      }
   }
}

//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
{
   double ma;
//---- go trading only for first tiks of new bar
   static datetime Time0;    
   
   if(Time0 == Time[0]) return(0); 
   
   Time0 = Time[0];
//---- get Moving Average 
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
   
   bool res = false; int retry = 0;
//----
   for(int i=OrdersTotal()-1; i>=0; i--)
   {      
      if(OrderSelect(i, SELECT_BY_POS) &&  OrderMagicNumber()  == MagicNumber &&  OrderSymbol() == Symbol() )
      {
      //---- check order type 
         if(OrderType()==OP_BUY)
         {
            if(Open[1]>ma && Close[1]<ma) 
            {
               while(!IsTradeAllowed()) Sleep(100);
               while(IsTradeContextBusy()) Sleep(100);
         
               while(retry < 5 && !res)
               {
                  RefreshRates();
                  res = OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),Slippage,White);
                  
                  Sleep(500);
                  retry++;
               }
               
               if(!res) Print(Symbol()," Unable to close buy trade! Error: ",GetLastError());
            }
         }
         if(OrderType()==OP_SELL)
         {
            if(Open[1]<ma && Close[1]>ma) 
            {
               while(!IsTradeAllowed()) Sleep(100);
               while(IsTradeContextBusy()) Sleep(100);
               
               while(retry < 5 && !res)
               {
                  RefreshRates();
               
                  res = OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),Slippage,White);
                  
                  Sleep(500);
                  retry++;
               }
               
               if(!res) Print(Symbol()," Unable to close sell trade! Error: ",GetLastError());
            }
         }
      }
   }
//----
}

//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
{
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
   
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) 
   {
      CheckForOpen();
   }
   else
   {
      Trailing_Stop() ;
      CheckForClose() ;
   }
   Comment("Forex Steady Growth status OK");
}
//+------------------------------------------------------------------+---+


void OpenBuy(string comment)
{
   int ticket = -1;
   int retry = 0;
   
   double SL,TP;
   
   if(StopLoss > 0) SL = Ask - StopLoss*Point;
   else SL = 0;
   if(TakeProfit > 0) TP = Ask + TakeProfit*Point;
   else TP = 0;
   
   while(!IsTradeAllowed()) Sleep(100);
   while(IsTradeContextBusy()) Sleep(100);
   
   while(ticket <= 0 && retry < 5)
   {
      RefreshRates();
      ticket = OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,Slippage,0,0,comment,MagicNumber,0,Blue);
      retry++;
      Sleep(200);
   }
   
   if(ticket > 0)
   {
      if(SL > 0 || TP > 0)
      {
         OrderSelect(ticket,SELECT_BY_TICKET);
         
         while(!IsTradeAllowed()) Sleep(50);
         while(IsTradeContextBusy()) Sleep(50);
         
         if(!OrderModify(OrderTicket(),OrderOpenPrice(),SL,TP,0,CLR_NONE)) 
         {
            Print(Symbol()," Unable to set SL/TP! Error: ",GetLastError());
            Print("Ask: ",Ask, " SL: ",SL, " TP: ",TP);
         }
      }
   }
   else Print(Symbol()," Unable to open buy trade! Error: ",GetLastError());
}

void OpenSell(string comment)
{
   int ticket = -1;
   int retry = 0;
   
   double SL,TP;
   
   if(StopLoss > 0) SL = Bid + StopLoss*Point;
   else SL = 0;
   if(TakeProfit > 0) TP = Bid - TakeProfit*Point;
   else TP = 0;
   
   while(!IsTradeAllowed()) Sleep(100);
   while(IsTradeContextBusy()) Sleep(100);
   
   while(ticket <= 0 && retry < 5)
   {
      RefreshRates();
      ticket = OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,Slippage,0,0,comment,MagicNumber,0,Red);
      retry++;
      Sleep(200);
   }
   
   if(ticket > 0)
   {
      if(SL > 0 || TP > 0)
      {
         OrderSelect(ticket,SELECT_BY_TICKET);
         
         while(!IsTradeAllowed()) Sleep(50);
         while(IsTradeContextBusy()) Sleep(50);
         
         if(!OrderModify(OrderTicket(),OrderOpenPrice(),SL,TP,0,CLR_NONE)) 
         {
            Print(Symbol()," Unable to set SL/TP! Error: ",GetLastError());
            Print("Bid: ",Bid, " SL: ",SL, " TP: ",TP);
         }
      }
   }
   else Print(Symbol()," Unable to open sell trade! Error: ",GetLastError());
}

int SymbolsList(string & Symbols[])
{
   int Offset, SymbolsNumber;
   
   string SymbolsFileName = "symbols.sel";
 
   int hFile = FileOpenHistory(SymbolsFileName, FILE_BIN|FILE_READ);
   if(hFile < 0) return(-1);
 
   SymbolsNumber = (FileSize(hFile)-4)/128; 
   Offset = 116;
   
   ArrayResize(Symbols, SymbolsNumber);
 
   FileSeek(hFile, 4, SEEK_SET);
   
   for(int i=0;i<SymbolsNumber;i++)
   {
      Symbols[i] = FileReadString(hFile, 12);
      FileSeek(hFile, Offset, SEEK_CUR);
   }
   
   FileClose(hFile);
 
   return(SymbolsNumber);
}

bool BrokerIs5Digit()
{
   string symbols[];
   string eurusd = "";
   
   if(SymbolsList(symbols) > 0)
   {
      for(int i=0;i<ArraySize(symbols);i++)
      {
         if(StringFind(symbols[i],"EURUSD",0) != -1) 
         {
            eurusd = symbols[i];
            break;
         }
      }
      
      if(eurusd != "") if(MarketInfo(eurusd,MODE_DIGITS)==5) return(true);
   }
   
   return(false);
}

